#include <stdio.h>
#include <stdarg.h>
#include "ctools.h"
#include "fileio.h"

static FILE*	fd_list[FDMAX];
static int		fd_flag[FDMAX];
static SOCKET	fd_sock[FDMAX];
static int		sockfd;

void fio_init(void) {
	int i;
	for(i=0;i<FDMAX;i++) {
		fd_list[i] = NULL;
		fd_sock[i] = NULL;
		fd_flag[i] = 0;
	}
	sockfd = -1;
	fd_list[STDIN_FILENO]  = stdin;
	fd_flag[STDIN_FILENO]  = 1;
	fd_list[STDOUT_FILENO] = stdout;
	fd_flag[STDOUT_FILENO] = 1;
	fd_list[STDERR_FILENO] = stderr;
	fd_flag[STDERR_FILENO] = 1;	
}

int fio_open(char *filename, char *mode) {
	int i;

	for(i=0;i<FDMAX;i++) {
		if (fd_flag[i] == 0) { break; }
	}
	if (i>=FDMAX) { return -1; }

	fd_flag[i] = 1;
	if ((fd_list[i] = fopen(filename,mode)) != NULL) {
		return i;
	} else {
		fd_flag[i] = 0;
		return -1;
	}
}

int fio_open_as_stdout(char *filename, char *mode) {
	int i;
	i = STDOUT_FILENO;

	fd_flag[i] = 1;
	if ((fd_list[i] = fopen(filename,mode)) != NULL) {
		return i;
	} else {
		fd_flag[i] = 0;
		return -1;
	}
}

int fio_opensocket(int port) {
	int r;
	int i;

	for(i=0;i<FDMAX;i++) {
		if (fd_flag[i] == 0) { break; }
	}
	if (i>=FDMAX) { return -1; }

	fd_flag[i] = 2;
	if (tcp_connect("127.0.0.1", port , &fd_sock[i])>=0) {
		sockfd = i;
		return i;
	} else {
		fd_flag[i] = 0;
		return -1;
	}
}

int fio_close(int fd) {
	int r;

	if (fd_flag[fd]==1) {
		r = fclose(fd_list[fd]);
		fd_flag[fd] = 0;
	} else if (fd_flag[fd]==2) {
		r = closesocket(fd_sock[fd]);
		fd_flag[fd] = 0;
	} else {
		r = -1;
	}
	return r;
}

ssize_t fio_read(int fd, void *buf, size_t count) {
	long s;

	if (fd_flag[fd]==1) {
		s = fread(buf, 1, count, fd_list[fd]);
	} else if (fd_flag[fd]==2) {
		s = recv(fd_sock[fd], buf, count, 0);
	} else {
		s = 0;
	}
	return s;
}

ssize_t fio_write(int fd, const void *buf, size_t count) {
	long s;

	if ((fd == STDOUT_FILENO) && (sockfd != -1)) { fd = sockfd; }
	if (fd_flag[fd]==1) {
		s = fwrite(buf, 1, count, fd_list[fd]);
	} else if (fd_flag[fd]==2) {
		s = send(fd_sock[fd], buf, count, 0);
	} else {
		s = 0;
	}
	return s;
}

off_t fio_lseek(int fd, off_t offset, int whence) {
	long s;

	if (fd_flag[fd]==1) {
		s = fseek(fd_list[fd], offset, whence);
		if (s == 0) { s = ftell(fd_list[fd]); }
	} else if (fd_flag[fd]==2) {
		s = 0;
	} else {
		s = 0;
	}
	return s;
}

void fio_fprintf(int fd,char *fmt, ...) {
	if (fd_flag[fd]==1) {
		va_list argp;
		va_start(argp, fmt);
		vfprintf(fd_list[fd], fmt, argp);
		va_end(argp);
	}
}



// -----------------------------------------------------------------------------
//   TCPڑ
// -----------------------------------------------------------------------------

int tcp_connect( char *hostname, int portno ,SOCKET *sock){
    IN_ADDR       inaddr;
    LPHOSTENT	  hostent;
    SOCKADDR_IN   sockaddr;
	int ret;

	inaddr.s_addr = inet_addr(hostname);
	if (inaddr.s_addr == INADDR_NONE) {
		hostent=gethostbyname(hostname);
	} else {
		hostent=gethostbyaddr((const char *)&inaddr, sizeof(struct in_addr), AF_INET);
	}
	if (hostent == NULL) { return -1; }
	*sock=socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (*sock==INVALID_SOCKET) { return -1; }
	sockaddr.sin_family = AF_INET;
	sockaddr.sin_port = htons(portno);    
	sockaddr.sin_addr = *((LPIN_ADDR)*hostent->h_addr_list);
	ret=connect(*sock, (LPSOCKADDR)&sockaddr, sizeof(SOCKADDR_IN));
	if (ret == SOCKET_ERROR) {
		closesocket(*sock);
		return -1;
	}
}

